# Generated by Django 2.1.3 on 2018-11-18 12:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('music_publisher', '0012_auto_20181117_1745'),
    ]

    operations = [
        migrations.AddField(
            model_name='cwrexport',
            name='_work_count',
            field=models.IntegerField(editable=False, null=True),
        ),
    ]
