# Generated by Django 2.1 on 2018-09-12 12:17

import django.core.validators
from django.db import migrations, models
import music_publisher.models


class Migration(migrations.Migration):

    dependencies = [
        ('music_publisher', '0008_auto_20180911_1055'),
    ]

    operations = [
        migrations.AddField(
            model_name='albumcd',
            name='album_label',
            field=models.CharField(blank=True, default='FOO BAR MUSIC', max_length=60, validators=[music_publisher.models.CWRFieldValidator('first_album_label')]),
        ),
        migrations.AlterField(
            model_name='albumcd',
            name='album_title',
            field=models.CharField(blank=True, default='', max_length=60, unique=True, validators=[music_publisher.models.CWRFieldValidator('first_album_title')]),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='workacknowledgement',
            name='society_code',
            field=models.CharField(choices=[('055', 'SABAM, Belgium'), ('101', 'SOCAN, Canada'), ('088', 'CMRRA, Canada'), ('040', 'KODA, Denmark'), ('089', 'TEOSTO, Finland'), ('058', 'SACEM, France'), ('035', 'GEMA, Germany'), ('074', 'SIAE, Italy'), ('023', 'BUMA, Netherlands'), ('078', 'STEMRA, Netherlands'), ('090', 'TONO, Norway'), ('079', 'STIM, Sweden'), ('052', 'PRS, United Kingdom'), ('044', 'MCPS, United Kingdom'), ('010', 'ASCAP, United States'), ('021', 'BMI, United States'), ('071', 'SESAC Inc., United States'), ('034', 'HFA, United States')], max_length=3, verbose_name='Society'),
        ),
        migrations.AlterField(
            model_name='writer',
            name='pr_society',
            field=models.CharField(blank=True, choices=[('055', 'SABAM, Belgium'), ('101', 'SOCAN, Canada'), ('088', 'CMRRA, Canada'), ('040', 'KODA, Denmark'), ('089', 'TEOSTO, Finland'), ('058', 'SACEM, France'), ('035', 'GEMA, Germany'), ('074', 'SIAE, Italy'), ('023', 'BUMA, Netherlands'), ('078', 'STEMRA, Netherlands'), ('090', 'TONO, Norway'), ('079', 'STIM, Sweden'), ('052', 'PRS, United Kingdom'), ('044', 'MCPS, United Kingdom'), ('010', 'ASCAP, United States'), ('021', 'BMI, United States'), ('071', 'SESAC Inc., United States'), ('034', 'HFA, United States')], help_text='Required for a controlled writer', max_length=3, null=True, validators=[music_publisher.models.CWRFieldValidator('writer_pr_society')], verbose_name='Performing Rights Society'),
        ),
        migrations.AlterField(
            model_name='writer',
            name='publisher_fee',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='Percentage of royalties kept by the publisher', max_digits=5, null=True, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(100)]),
        ),
        migrations.AlterField(
            model_name='writerinwork',
            name='publisher_fee',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='Percentage of royalties kept by the publisher', max_digits=5, null=True, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(100)]),
        ),
        migrations.AlterUniqueTogether(
            name='albumcd',
            unique_together={('album_title', 'album_label')},
        ),
    ]
