"""
This app is targeted at single original publishers, publishing original musical
works. It holds data on musical works, including alternate titles, songwriters,
performing artists, first recordings, music library and albums. It allows data
imports and exports.

Multiple writers, both controlled and uncontrolled, are covered, with minor
limitations, but data on other publishers (other original publishers,
administrators and sub-publishers) can not be entered.

A special US situation where an original publisher may have one entity for
every of the three PROs is also covered.
"""
