# Generated by Django 2.1.3 on 2018-11-14 15:48

from django.db import migrations, models
import music_publisher.base


class Migration(migrations.Migration):

    dependencies = [
        ('music_publisher', '0009_auto_20180912_1217'),
    ]

    operations = [
        migrations.AddField(
            model_name='artist',
            name='isni',
            field=models.CharField(blank=True, null=True, max_length=16, validators=[music_publisher.base.CWRFieldValidator('isni')]),
        ),
        migrations.AlterField(
            model_name='workacknowledgement',
            name='society_code',
            field=models.CharField(choices=[('055', 'SABAM, Belgium'), ('101', 'SOCAN, Canada'), ('088', 'CMRRA, Canada'), ('040', 'KODA, Denmark'), ('089', 'TEOSTO, Finland'), ('058', 'SACEM, France'), ('035', 'GEMA, Germany'), ('074', 'SIAE, Italy'), ('023', 'BUMA, Netherlands'), ('078', 'STEMRA, Netherlands'), ('090', 'TONO, Norway'), ('079', 'STIM, Sweden'), ('052', 'PRS, United Kingdom'), ('044', 'MCPS, United Kingdom'), ('010', 'ASCAP, United States'), ('021', 'BMI, United States'), ('071', 'SESAC Inc., United States'), ('034', 'HFA, United States'), ('319', 'ICE Services, Administrative Agency'), ('707', 'Musicmark, Administrative Agency')], max_length=3, verbose_name='Society'),
        ),
        migrations.AlterField(
            model_name='workacknowledgement',
            name='status',
            field=models.CharField(choices=[('CO', 'Conflict'), ('DU', 'Duplicate'), ('RA', 'Transaction Accepted'), ('AS', 'Registration Accepted'), ('AC', 'Registration Accepted with Changes'), ('RJ', 'Rejected'), ('NP', 'No Participation'), ('RC', 'Claim rejected')], max_length=2),
        ),
        migrations.AlterField(
            model_name='writer',
            name='ipi_name',
            field=models.CharField(blank=True, max_length=11, null=True, unique=True, validators=[music_publisher.base.CWRFieldValidator('writer_ipi_name')], verbose_name='IPI Name #'),
        ),
        migrations.AlterField(
            model_name='writer',
            name='pr_society',
            field=models.CharField(blank=True, choices=[('055', 'SABAM, Belgium'), ('101', 'SOCAN, Canada'), ('088', 'CMRRA, Canada'), ('040', 'KODA, Denmark'), ('089', 'TEOSTO, Finland'), ('058', 'SACEM, France'), ('035', 'GEMA, Germany'), ('074', 'SIAE, Italy'), ('023', 'BUMA, Netherlands'), ('078', 'STEMRA, Netherlands'), ('090', 'TONO, Norway'), ('079', 'STIM, Sweden'), ('052', 'PRS, United Kingdom'), ('044', 'MCPS, United Kingdom'), ('010', 'ASCAP, United States'), ('021', 'BMI, United States'), ('071', 'SESAC Inc., United States'), ('034', 'HFA, United States'), ('319', 'ICE Services, Administrative Agency'), ('707', 'Musicmark, Administrative Agency')], max_length=3, null=True, validators=[music_publisher.base.CWRFieldValidator('writer_pr_society')], verbose_name='Performing Rights Society'),
        ),
    ]
