# Generated by Django 2.1.3 on 2018-11-18 18:54

from django.db import migrations, models
import music_publisher.base


class Migration(migrations.Migration):

    dependencies = [
        ('music_publisher', '0013_cwrexport__work_count'),
    ]

    operations = [
        migrations.AlterField(
            model_name='albumcd',
            name='album_title',
            field=models.CharField(blank=True, max_length=60, null=True, unique=True, validators=[music_publisher.base.CWRFieldValidator('first_album_title')]),
        ),
        migrations.AlterField(
            model_name='cwrexport',
            name='works',
            field=models.ManyToManyField(related_name='cwr_exports', to='music_publisher.Work'),
        ),
    ]
