# Generated by Django 2.1 on 2018-09-11 10:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('music_publisher', '0007_auto_20180911_1017'),
    ]

    operations = [
        migrations.AddField(
            model_name='writer',
            name='publisher_fee',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='Percentage of royalties kept by the publisher', max_digits=5, null=True),
        ),
        migrations.AddField(
            model_name='writerinwork',
            name='publisher_fee',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='Percentage of royalties kept by the publisher', max_digits=5, null=True),
        ),
        migrations.AlterField(
            model_name='writer',
            name='generally_controlled',
            field=models.BooleanField(default=False, verbose_name='General Agreement'),
        ),
    ]
