# Generated by Django 2.1.3 on 2018-11-17 17:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('music_publisher', '0011_auto_20181114_1649'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='artistinwork',
            options={'verbose_name': 'Artist performing work', 'verbose_name_plural': 'Artists performing works'},
        ),
        migrations.AlterField(
            model_name='cwrexport',
            name='nwr_rev',
            field=models.CharField(choices=[('NWR', 'CWR 2.1: New work registration'), ('REV', 'CWR 2.1: Revision of registered work'), ('WRK', 'CWR 3.0: Work registration (alpha)')], db_index=True, default='NWR', max_length=3, verbose_name='NWR/REV'),
        ),
    ]
