# Generated by Django 2.1.3 on 2018-11-14 16:49

from django.db import migrations, models
import django.db.models.deletion
import music_publisher.base


class Migration(migrations.Migration):

    dependencies = [
        ('music_publisher', '0010_auto_20181114_1548'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='firstrecording',
            name='catalog_number',
        ),
        migrations.AddField(
            model_name='firstrecording',
            name='artist',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='music_publisher.Artist', verbose_name='Recording Artist'),
        ),
        migrations.AddField(
            model_name='firstrecording',
            name='record_label',
            field=models.CharField(blank=True, default='FOO BAR MUSIC', max_length=60, validators=[music_publisher.base.CWRFieldValidator('first_album_label')]),
        ),
        migrations.AlterField(
            model_name='artist',
            name='isni',
            field=models.CharField(blank=True, max_length=16, null=True, unique=True, validators=[music_publisher.base.CWRFieldValidator('isni')], verbose_name='ISNI'),
        ),
    ]
